#include <SPI.h>
#include <Ethernet.h>

#include "WebServer.h"
#include "PZEM004T.h"
#include "SoftwareSerial.h"

#define RETRIES_BEFORE_ERROR 20

// assign a MAC address for the Ethernet controller.
// fill in your address here:
byte mac[] = {
  0xDE, 0xBD, 0xBE, 0xEF, 0xF2, 0x3D
};
// assign an IP address for the controller:
IPAddress ip(192, 168, 2, 223);


// Initialize the Ethernet server library
// with the IP address and port you want to use
// (port 80 is default for HTTP):
WebServer server("", 80);

PZEM004T pzem(11,12); // RX=0,TX=1
IPAddress pzemIP(192, 168, 1, 1);

long lastReadingTime = 0;
float voltage = 0;
float current = 0;
float power = 0;
float energy = 0;

uint8_t retries = 0;
uint8_t indicator = 0;

void defaultCmd(WebServer &server, WebServer::ConnectionType type, char *url_tail, bool tail_complete)
{
  if (type == WebServer::POST)
  {
    server.httpFail();
    return;
  }

  //server.httpSuccess(false, "application/json");
  server.httpSuccess("application/json");

  if (type == WebServer::HEAD)
    return;

  server.println("{");
  server.print("  \"voltage\": " + String(voltage));
  server.println(",");
  server.print("  \"current\": " + String(current));
  server.println(",");
  server.print("  \"power\": " + String(power));
  server.println(",");
  server.println("  \"energy\": " + String(energy));
  server.println("}");
}

void setup() {
  pzem.setReadTimeout(50);
  pzem.setAddress(pzemIP);

  // start the SPI library:
  SPI.begin();

  // start the Ethernet connection and the server:
  Ethernet.begin(mac, ip);

  server.begin();
  server.setDefaultCommand(&defaultCmd);
}

void check() {
  if (indicator == 4)
    indicator = 0;

  if (indicator == 0) {
    float v = pzem.voltage(pzemIP);
    if (v >= 0.0) {
      voltage = v;
      indicator++;
      retries = 0;
    } else {
      if (++retries > RETRIES_BEFORE_ERROR)
        voltage = 0.0;
    }
  } else if (indicator == 1) {
    float i = pzem.current(pzemIP);
    if (i >= 0.0) {
      current = i;
      indicator++;
      retries = 0;
    } else {
      if (++retries > RETRIES_BEFORE_ERROR)
        current = 0.0;
    }
  } else if (indicator == 2) {
    float p = pzem.power(pzemIP);
    if (p >= 0.0) {
      power = p;
      indicator++;
      retries = 0;
    } else {
      if (++retries > RETRIES_BEFORE_ERROR)
        power = 0.0;
    }
  } else if (indicator == 3) {
    float e = pzem.energy(pzemIP);
    if (e >= 0.0) {
      energy = e;
      indicator++;
      retries = 0;
    } else {
      if (++retries > RETRIES_BEFORE_ERROR)
        indicator++;
    }
  }
}

void loop() {
  check();

  // process incoming connections one at a time forever
  server.processConnection();
}

