#ifndef PZEM004T_H
#define PZEM004T_H

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

// #define PZEM004_NO_SWSERIAL

#if (not defined(PZEM004_NO_SWSERIAL)) && (defined(__AVR__) || defined(ESP8266))
#define PZEM004_SOFTSERIAL
#endif

#if defined(PZEM004_SOFTSERIAL)
#include <SoftwareSerial.h>
#endif

#include <IPAddress.h>

#define PZEM_VOLTAGE 1
#define PZEM_CURRENT 2
#define PZEM_POWER   4
#define PZEM_ENERGY  8
#define PZEM_ALL_VALUES (PZEM_VOLTAGE | PZEM_CURRENT | PZEM_POWER | PZEM_ENERGY)

#define PZEM_DEFAULT_READ_TIMEOUT 1000
#define PZEM_ERROR_VALUE -1.0

#define PZEM004_RESPONSE_SIZE sizeof(PZEMCommand)
#define PZEM004_RESPONSE_DATA_SIZE PZEM004_RESPONSE_SIZE - 2

class PZEM004T
{
public:
    PZEM004T(uint8_t receivePin, uint8_t transmitPin);
    PZEM004T(HardwareSerial *port);
    ~PZEM004T();

    void setReadTimeout(unsigned long msec);
    unsigned long readTimeout();

    float voltage(const IPAddress &addr = IPAddress());
    float current(const IPAddress &addr = IPAddress());
    float power(const IPAddress &addr = IPAddress());
    float energy(const IPAddress &addr = IPAddress());

    bool setAddress(const IPAddress &newAddr);
    const IPAddress &address();

    bool setPowerAlarm(const IPAddress &addr, uint8_t threshold);

    void setNonBlocking(bool nonbl);
    bool nonBlocking();

    void update(uint8_t values = PZEM_ALL_VALUES, const IPAddress &addr = IPAddress());

private:
    struct PZEMCommand {
        uint8_t command;
        uint8_t addr[4];
        uint8_t data;
        uint8_t crc;
    } _pzemCommand;

    Stream *serial;
    IPAddress _address;

    bool _isSoft;
    unsigned long requestTime;
    unsigned long _readTimeOut = PZEM_DEFAULT_READ_TIMEOUT;
    bool _nonBlocking          = false;

    uint8_t currentValue = 0;
    float _voltage = PZEM_ERROR_VALUE;
    float _current = PZEM_ERROR_VALUE;
    float _power   = PZEM_ERROR_VALUE;
    float _energy  = PZEM_ERROR_VALUE;

    uint8_t _buffer[PZEM004_RESPONSE_SIZE];
    uint8_t _dataBuffer[PZEM004_RESPONSE_DATA_SIZE];

    bool waitResponse();
    bool readResponse(uint8_t resp);

    void send(const IPAddress &addr, uint8_t cmd, uint8_t data = 0);
    bool recieve(uint8_t resp);

    uint8_t crc(uint8_t *data, uint8_t sz);
};

#endif // PZEM004T_H
